@@ -77,7 +77,7 @@ drgeeData <-
 
         if (!missing(oformula)) {
             oterms <- terms(oformula)
-            omf <- model.frame(formula = oformula, data = data, na.action = na.pass)
+            omf <- model.frame(formula = oformula, data = data, na.action = na.pass, drop.unused.levels = TRUE)
 
             ## we define the outcome as the response in the oformula
             if (attr(oterms,"response")) {
@@ -125,7 +125,7 @@ drgeeData <-
                 environment(tempof) <- data
             }
 
-            yf <- model.frame(tempof, data = data, na.action = na.pass)
+            yf <- model.frame(tempof, data = data, na.action = na.pass, drop.unused.levels = TRUE)
             y.all <- model.matrix(tempof, yf)
 
             if (dim(y.all)[2] != 2) {
@@ -159,11 +159,10 @@ or a factor with two levels\n\n")
             }
         }
 
-
         if (!missing(eformula)) {
 
             eterms <- terms(eformula)
-            emf <- model.frame(formula = eformula, data = data, na.action = na.pass)
+            emf <- model.frame(formula = eformula, data = data, na.action = na.pass, drop.unused.levels = TRUE)
 
             if (!length(attr(eterms, "term.labels")) == 0 & nrow(emf) != nobs) {
                 stop("The should be as many outcome as exposure nuisance model observations\n\n")
@@ -193,7 +192,6 @@ or a factor with two levels\n\n")
         ## Extract the exposure if it is given
         if (!missing(exposure)) {
 
-
             ## If the exposure is not given as a string
             ## get the name of the object that was given
             ## as input
@@ -218,7 +216,7 @@ or a factor with two levels\n\n")
                 environment(tempef) <- data
             }
 
-            af <- model.frame(tempef, data = data, na.action = na.pass)
+            af <- model.frame(tempef, data = data, na.action = na.pass, drop.unused.levels = TRUE)
             a.all <- model.matrix(tempef, af)
 
             if (dim(a.all)[2] != 2) {
@@ -229,7 +227,7 @@ or a factor with two levels\n\n")
             a <- a.all[, 2, drop = F]
 
             ## Update the column names if the outcome was not numeric
-            expname <- colnames(a)
+            a.names <- colnames(a)
 
             if ( nrow(a) != nobs) {
                 stop("The should be as many outcome as exposure observations\n\n")
@@ -246,13 +244,14 @@ or a factor with two levels\n\n")
 
         } else {
             a <- NULL
+            a.names <- NULL
         }
 
         if (!missing(iaformula)) {
 
             iaterms <- terms(iaformula)
             iamf <- model.frame(formula = iaformula, data = data,
-                                na.action = na.pass)
+                                na.action = na.pass, drop.unused.levels = TRUE)
 
             if (length(attr(iaterms, "term.labels")) == 0) {
                 iaterms <- NULL
@@ -318,32 +317,6 @@ or a factor with two levels\n\n")
             obs.order <- 1:nobs
         }
 
-        ## For conditional logistic methods, we exclude non-informative clusters
-        if (cond & olink == "logit") {
-            if (estimation.method == "o") {
-
-                id.tmp <- as.factor(id[obs.order])
-                y.tmp <- y[obs.order, 1]
-                ## For each cluster, identify the number of different values for the outcome
-                ni.vals <- ave(as.vector(y.tmp), id.tmp, FUN = function(y) {length(unique(y[which(!is.na(y)) ]))})
-
-                ## Only use outcome-discordant clusters
-                compl.rows <- compl.rows & (ni.vals > 1)
-
-            } else if (estimation.method == "e") {
-
-                id.tmp <- as.factor(id[obs.order])
-                a.tmp <- a[obs.order, 1]
-                ## For each cluster, identify the number of different values for the outcome
-                ni.vals <- ave(as.vector(a.tmp), id.tmp, FUN = function(a) {length(unique(a[which(!is.na(a))]))})
-
-                ## Only use outcome-discordant clusters
-                compl.rows <- compl.rows & (ni.vals > 1)
-
-            }
-        }
-
-
         nobsnew <- sum(compl.rows)
 
         ## Get the row numbers that we will use
@@ -352,6 +325,7 @@ or a factor with two levels\n\n")
         ## Extract observations for complete observations
         id <- as.factor(id[rows.to.use])
         y <- y[rows.to.use,, drop = F]
+        y.names <- colnames(y)
 
         if (!is.null(a)) {
             a <- a[rows.to.use,, drop = F]
@@ -359,23 +333,19 @@ or a factor with two levels\n\n")
 
         ## Extract covariates
         if (!is.null(oterms)) {
-            if (length(attr(oterms, "term.labels")) > 0) {
-                v <- model.matrix(oterms, omf)[rows.to.use,, drop = F]
-            } else {
-                v <- matrix( rep(1, nobsnew), ncol = 1)
-            }
+            v <- model.matrix(oterms, omf)[rows.to.use,, drop = F]
+            v.names <- colnames(v)
         } else {
             v <- NULL
+            v.names <- NULL
         }
 
         if (!is.null(eterms)) {
-            if (length(attr(eterms, "term.labels")) > 0) {
-                z <- model.matrix(eterms, emf)[rows.to.use,, drop = F]
-            } else {
-                z <- matrix( rep(1, nobsnew), ncol = 1)
-            }
+            z <- model.matrix(eterms, emf)[rows.to.use,, drop = F]
+            z.names <- colnames(z)
         } else {
             z <- NULL
+            z.names <- NULL
         }
 
         x <- matrix( rep(1, nobsnew), ncol = 1)
@@ -386,20 +356,28 @@ or a factor with two levels\n\n")
             }
         }
 
+        x.names <- colnames(x)
+        
         if (olink == "logit") {
+            yx.names <- rep("", ncol(x))
+            yx.names[1] <- y.names
+            yx.names[-1] <- paste(y.names, x.names[-1], sep = ":")
             yx <- x * as.vector(y)
-            colnames(yx)[1] <- colnames(y)
-            colnames(yx)[-1] <- paste(colnames(y), colnames(x)[-1], sep = ":")
+            dimnames(yx) <- list(rownames(yx), yx.names)
         } else {
             yx <- NULL
+            yx.names <- NULL
         }
 
         if (!is.null(a)) {
+            ax.names <- rep("", ncol(x))
+            ax.names[1] <- a.names
+            ax.names[-1] <- paste(a.names, x.names[-1], sep = ":")
             ax <- x * as.vector(a)
-            colnames(ax)[1] <- colnames(a)
-            colnames(ax)[-1] <- paste(colnames(a), colnames(x)[-1], sep = ":")
+            dimnames(ax) <- list(rownames(ax), ax.names)
         } else {
             ax <- NULL
+            ax.names <- NULL
         }
 
         ## Do not use an intercept for conditional methods
@@ -408,29 +386,48 @@ or a factor with two levels\n\n")
             if (!is.null(oterms)) {
                 if (length(attr(oterms, "term.labels"))>0 & attr(oterms,"intercept")) {
                     v <- v[, -1, drop = F]
+                    v.names = colnames(v)
                 } else {
                     v <- NULL
+                    v.names <- NULL
                 }
             }
 
             if (!is.null(eterms)) {
                 if (length(attr(eterms, "term.labels"))>0 & attr(eterms,"intercept")) {
                     z <- z[, -1, drop = F]
+                    z.names = colnames(z)
+
                 } else {
                     z <- NULL
+                    z.names <- NULL
                 }
             }
 
 
         }
 
-        drgee.data <- list(y = y, a = a, x = x, ax = ax, v = v, z = z, yx = yx,
-                          id = id, idname = clustname,
-                          outname = outname, expname = expname,
-                          clustname = clustname,
-                          olink = olink, elink = elink,
-                          oterms = oterms, eterms = eterms,
-                          cond = cond)
+        drgee.data <- list(y = y,
+                           a = a,
+                           x = x,
+                           ax = ax,
+                           v = v,
+                           z = z,
+                           yx = yx,
+                           id = id,
+                           clustname = clustname,
+                           y.names = y.names, 
+                           a.names = a.names, 
+                           x.names = x.names, 
+                           ax.names = ax.names, 
+                           v.names = v.names, 
+                           z.names = z.names, 
+                           yx.names = yx.names, 
+                           olink = olink,
+                           elink = elink,
+                           cond = cond, 
+                           oterms = oterms,
+                           eterms = eterms )
 
         class(drgee.data) <- "drgeeData"
         return(drgee.data)
@@ -439,57 +436,70 @@ or a factor with two levels\n\n")
 summary.drgeeData <-
     function(object, ...) {
 
-        outcome <- colnames(object$y)
-        exposure <- colnames(object$a)
-        exposures.all <- colnames(object$ax)
-        covariates <- setdiff( union(colnames(object$v),colnames(object$z)),  "(Intercept)" )
+        covariates <- setdiff( union(object$v.names, object$z.names),
+                              "(Intercept)" )
+        
+        if(length(covariates) == 0) {
+            covariates <- "None"
+        }
+
         interactions <- colnames(object$x)
 
-        main.model <- paste(colnames(object$y), "~",
-                            paste(colnames(object$ax), collapse = " + "), sep = " ")
+        main.model <- paste(object$y.names, "~",
+                            paste(object$ax.names, collapse = " + "), sep = " ")
 
         if (object$cond) {
-            outcome.nuisance.model <- paste(colnames(object$y),"~",
-                                            paste(c(colnames(object$v)),
+            outcome.nuisance.model <- paste(object$y.names,"~",
+                                            paste(c(object$v.names),
                                                   collapse = " + "), sep = " ")
-            outcome.model <- paste(colnames(object$y),"~",
-                                   paste(c(colnames(object$ax), colnames(object$v)),
+            outcome.model <- paste(object$y.names,"~",
+                                   paste(c(object$ax.names, object$v.names),
                                          collapse = " + "), sep = " ")
 
         } else {
-            outcome.nuisance.model <- paste(colnames(object$y),"~",
-                                            paste(c(colnames(object$v)[-1]),
-                                                  collapse = " + "), sep = " ")
-            outcome.model <- paste(colnames(object$y),"~",
-                                   paste(c(colnames(object$ax), colnames(object$v)[-1]),
+            if (!is.null(object$v)) {
+                outcome.nuisance.model <- paste(object$y.names,"~",
+                                                paste(c(object$v.names[-1]),
+                                                      collapse = " + "), sep = " ")
+            } else {
+                outcome.nuisance.model <- paste(object$y.names,"~ 1")
+            }
+
+            outcome.model <- paste(object$ynames,"~",
+                                   paste(c(object$ax.names, object$v.names[-1]),
                                          collapse = " + "), sep = " ")
 
         }
 
         if (object$cond) {
-            exposure.nuisance.model <- paste(colnames(object$a),"~",
-                                             paste(colnames(object$z),
+            exposure.nuisance.model <- paste(object$a.names,"~",
+                                             paste(object$z.names,
                                                    collapse = " + "), sep = " ")
 
-            exposure.model <- paste(colnames(object$a),"~",
-                                    paste(c(colnames(object$yx), colnames(object$z)),
+            exposure.model <- paste(object$a.names,"~",
+                                    paste(c(object$yx.names, object$z.names),
                                           collapse = " + "), sep = " ")
 
         } else {
-            exposure.nuisance.model <- paste(colnames(object$a),"~",
-                                             paste(colnames(object$z)[-1],
-                                                   collapse = " + "), sep = " ")
 
-            exposure.model <- paste(colnames(object$a),"~",
-                                    paste(c(colnames(object$yx), colnames(object$z)[-1]),
+            if(!is.null(object$z)){
+                exposure.nuisance.model <- paste(object$a.anmes,"~",
+                                                 paste(object$z.names[-1],
+                                                       collapse = " + "), sep = " ")
+            } else {
+                exposure.nuisance.model <- paste(object$a.names,"~ 1")
+            }
+
+            exposure.model <- paste(object$a.names,"~",
+                                    paste(c(object$yx.names, object$z.names[-1]),
                                           collapse = " + "), sep = " ")
 
         }
 
-        ans <- list(outcome = outcome,
-                    exposure = exposure,
+        ans <- list(outcome = object$y.names,
+                    exposure = object$a.names,
                     covariates = covariates,
-                    interactions = interactions,
+                    interactions = object$x.names,
                     main.model = main.model,
                     outcome.nuisance.model = outcome.nuisance.model,
                     outcome.model = outcome.model,
@@ -511,8 +521,8 @@ print.summary.drgeeData <-
     function(x, digits = max(3L, getOption("digits") -
                     3L), ...) {
 
-        cat("\nOutcome: ", colnames(x$outcome), "\nExposure: ", colnames(x$exposure),
-            "\nInteractions: ", paste(colnames(x$interactions), collapse = ", "))
+        cat("\nOutcome: ", x$outcome, "\nExposure: ", x$exposure,
+            "\nInteractions: ", paste(x$interactions, collapse = ", "))
 
         cat("\n\nMain model: ", x$main.model, "\nwith link function: ", x$olink)
 
